open Core_c
open Highgui_c
open Imgproc_c
open Video_c
#load "unix.cma"

let pyr_scale   = 0.25  (* 0.5 *)
let levels      = 1     (* 3 *)
let winsize     = 20    (* 15 *)
let iterations  = 1     (* 3 *)
let poly_n      = 7     (* 5 *)
let poly_sigma  = 1.5   (* 1.2 *)
let flags       = 0     (* 0 *)

external mat_of_img : Core_c.image -> Core_c.cv_mat = "%identity"
external img_of_mat : Core_c.cv_mat -> Core_c.image = "%identity"
external arr_of_mat : Core_c.cv_mat -> Core_c.cv_arr = "%identity"
external mat_of_arr : Core_c.cv_arr -> Core_c.cv_mat = "%identity"

let fold_step a b step acc f =
  let rec aux i acc =
    if i < b then aux (i + step) (f i acc)
    else acc
  in
  aux a acc

let round x =
  int_of_float (floor (x +. 0.5))

let index cols rows x y =
  match Sys.argv.(1) with
  | "A" -> rows * x + y
  | "B" -> rows * y + x
  | "C" -> cols * x + y
  | "D" -> cols * y + x
  | _ -> invalid_arg "index"

let cv_mat_elem mat x y =
  let cols, rows = cv_mat_dims mat in
  assert (y < rows);
  assert (x < cols);
  let i = (index cols rows x y) * 2 in
  (cvMat_get_float mat Float i,
   cvMat_get_float mat Float (i+1))

let drawOptFlowMap ~flow ~img ~dt ~step ~color =
  let cols, rows = cv_mat_dims img in
  let x, y =
    fold_step 0 (pred rows) step (0.0, 0.0) (fun y acc_dir ->
      fold_step 0 (pred cols) step acc_dir (fun x (acc_x, acc_y) ->
        let fx, fy = cv_mat_elem flow x y in
        let dx = round (fx *. 2.0) in
        let dy = round (fy *. 2.0) in
        cvLine (arr_of_mat img) (x, y) (x+dx, y+dy) color 1 8 0;
        (acc_x +. fx, acc_y +. fy)
      )
    )
  in
  let cols, rows = (float cols, float rows) in
  Printf.printf "# %+d %+d\n%!"
    (round (x /. cols *. 100.0))
    (round (y /. rows *. 100.0))

let () =
  let capture = cvCreateCameraCapture 0 in
  cvNamedWindow "mywindow" Window_AutoSize;
  let frame = cvQueryFrame capture in
  let width, height = image_dims frame in
  let s_width, s_height = (width/4, height/4) in
  let gray = cvCreateMat height width CV_8UC1 in
  let small = cvCreateMat s_height s_width CV_8UC1 in
  let prev0 = cvCreateMat s_height s_width CV_8UC1 in
  let flow = cvCreateMat s_height s_width CV_32FC2 in
  let n = ref 0 in
  let t0 = Unix.gettimeofday () in
  let rec loop (prev, small) tp =
    let t = Unix.gettimeofday () in
    let dt = t -. tp in
    let frame = cvQueryFrame capture in
    cvCvtColor (mat_of_img frame) gray CV_BGR2GRAY;
    cvResize (arr_of_mat gray) (arr_of_mat small) CV_INTER_LINEAR;
    cvFlip (arr_of_mat small) (arr_of_mat small) 1;
    cvCalcOpticalFlowFarneback
      (arr_of_mat prev) (arr_of_mat small) (arr_of_mat flow)
      pyr_scale levels winsize iterations poly_n poly_sigma flags;
    drawOptFlowMap flow small dt 26 (rgb (255.0, 0.0, 0.0));
    cvShowImage "mywindow" (img_of_mat small);
    incr n;
    if ((cvWaitKey 100) land 255) <> 27
    then loop (small, prev) t
  in
  loop (prev0, small) t0;
  let t1 = Unix.gettimeofday () in
  cvReleaseCapture capture;
  cvDestroyWindow "mywindow";
  Printf.printf "fps: %f\n" (float !n /. (t1 -. t0));
;;
