open Core_c
open Highgui_c
open Imgproc_c
open Video_c

let help () =
  Printf.printf
    "\n This program demonstrate dense \"Farneback\n optical flow\n\
     It read from camera 0, and shows how to use and display \
     dense Franeback optical flow\n\
     Usage: \n\
     ./%s" Sys.argv.(0)

let for_step a b step f =
  let rec aux i =
    if i < b then ( f i; aux (i + step) )
  in
  aux a

let cv_mat_elem mat x y =
  let cols, rows = cv_mat_dims mat in
  assert (y < rows);
  assert (x < cols);
  let pix_size = 8 in
  let step = cvMat_get_step mat in
  let i = step * y + x in
  (cvMat_get_float mat Float i,
   cvMat_get_float mat Float (i+1))

let round x = int_of_float (floor(x +. 0.5))

let drawOptFlowMap ~flow ~cflowmap ~step ~color =
  let cols, rows = cv_mat_dims cflowmap in
  for_step 0 (pred rows) step (fun y ->
    for_step 0 (pred cols) step (fun x ->
      let fx, fy = cv_mat_elem mat x y in
      let fx = round fx in
      let fy = round fy in
      cvLine cflowmap (x, y) (point (x + fx) (y + fy)) color 1 8 0;
      (*
      CvPoint2D32f fxy = CV_MAT_ELEM( *flow, CvPoint2D32f, y, x);
      cvLine(cflowmap, cvPoint(x,y), cvPoint(cvRound(x+fxy.x), cvRound(y+fxy.y)),
           color, 1, 8, 0);
      cvCircle(cflowmap, cvPoint(x,y), 2, color, -1, 8, 0);
      *)
      ()
    )
  )

external mat_of_img : Core_c.image -> Core_c.cv_mat = "%identity"
external img_of_mat : Core_c.cv_mat -> Core_c.image = "%identity"
external arr_of_mat : Core_c.cv_mat -> Core_c.cv_arr = "%identity"
external mat_of_arr : Core_c.cv_arr -> Core_c.cv_mat = "%identity"


let () =
  let capture = cvCreateCameraCapture 0 in
  help ();
  cvNamedWindow "flow" Window_AutoSize;

  let frame = cvQueryFrame capture in
  let width, height = image_dims frame in

  let gray = cvCreateMat height width CV_8UC1 in
  let half_gray = cvCreateMat (height/3) (width/3) CV_8UC1 in
  let cols, rows = cv_mat_dims half_gray in
  (*
  let cv_type = get_cv_mat_type half_gray in
  let prev = cvCreateMat rows cols cv_type in
  *)
  let prev = cvCreateMat rows cols CV_8UC1 in

  let flow = cvCreateMat rows cols CV_32FC2 in
  let cflow = cvCreateMat rows cols CV_8UC3 in

  let rec loop prev firstFrame =
    let frame = cvQueryFrame capture in

    cvCvtColor (mat_of_img frame) gray CV_BGR2GRAY;
    cvResize (arr_of_mat gray) (arr_of_mat half_gray) CV_INTER_LINEAR;
    cvFlip (arr_of_mat half_gray) (arr_of_mat half_gray) 1;

  (*  {{{ doc for cvCalcOpticalFlowFarneback
      cvCalcOpticalFlowFarneback(
          const CvArr* prev, const CvArr* next, CvArr* flow,
          double pyr_scale, int levels, int winsize,
          int iterations, int poly_n, double poly_sigma, int flags)
  
      flow : computed flow image that has the same size as prev and type CV_32FC2.
      pyr_scale : parameter, specifying the image scale (<1) to build pyramids
                  for each image; pyr_scale=0.5 means a classical pyramid, where 
                  each next layer is twice smaller than the previous one.
      levels : number of pyramid layers including the initial image; levels=1 means
          that no extra layers are created and only the original images are used.
      winsize : averaging window size; larger values increase the algorithm robustness
          to image noise and give more chances for fast motion detection, but yield more
          blurred motion field.
      iterations : number of iterations the algorithm does at each pyramid level.
      poly_n : size of the pixel neighborhood used to find polynomial expansion in each pixel;
          larger values mean that the image will be approximated with smoother surfaces, yielding
          more robust algorithm and more blurred motion field, typically poly_n =5 or 7.
      poly_sigma : standard deviation of the Gaussian that is used to smooth derivatives
          used as a basis for the polynomial expansion; for poly_n=5, you can set
          poly_sigma=1.1, for poly_n=7, a good value would be poly_sigma=1.5.
      flags : operation flags that can be a combination of the following:
        OPTFLOW_USE_INITIAL_FLOW uses the input flow as an initial flow approximation.
        OPTFLOW_FARNEBACK_GAUSSIAN uses the Gaussian \texttt{winsize}\times\texttt{winsize}
          filter instead of a box filter of the same size for optical flow estimation;
          usually, this option gives z more accurate flow than with a box filter, at the cost
          of lower speed; normally, winsize for a Gaussian window should be set to a larger
          value to achieve the same level of robustness.
      }}} *)
    if not firstFrame then begin
      let pyr_scale = 0.25 in (* 0.5 *)
      let levels = 1 in (* 3 *)
      let winsize = 20 in (* 15 *)
      let iterations = 2 in (* 3 *)
      let poly_n = 7 in (* 5 *)
      let poly_sigma = 1.5 in (* 1.2 *)
      let flags = 0 in (* 0 *)
      cvCalcOpticalFlowFarneback
         prev (arr_of_mat half_gray) (arr_of_mat flow) pyr_scale levels
         winsize iterations poly_n poly_sigma flags;
      (*
      cvCvtColor (mat_of_arr prev) cflow CV_GRAY2BGR;
      drawOptFlowMap flow cflow 22 CV_RGB(255, 0, 0);
      cvShowImage "flow" (img_of_mat cflow);
      *)
      cvShowImage "flow" (img_of_mat half_gray);
    end;
    (*
    if (cvWaitKey 1) < 0 then
    *)
    loop (arr_of_mat half_gray) false
  in
  loop (arr_of_mat prev) true;
  cvReleaseCapture capture;
;;

(* vim: set fdm=marker: *)
