#include <opencv2/video/tracking.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <stdio.h>
#include <math.h>

#define SHOW_IMG 1
#define X_WARP_PTR 0

#if X_WARP_PTR
#include <X11/Xlib.h>
#include <X11/Xutil.h>
int screen_w;
int screen_h;
Display *display;
Window root;
int screen_number;
#endif

void drawOptFlowMap(
        const CvMat* flow, CvMat* cflowmap, int step, CvScalar color)
{
    double _cols = cflowmap->cols;
    double _rows = cflowmap->rows;
    int x, y;
    double ax, ay;
    double rx, ry;
    double len, lens = 0.0;
    static double cx = -1.0;
    static double cy = -1.0;
    static double sx = 0.0;
    static double sy = 0.0;
    double sn;
    double max_len = 0.0;
    if (cx == -1.0 && cy == -1.0) {
        cx = cflowmap->cols * 0.5;
        cy = cflowmap->rows * 0.5;
        sx = cx;
        sy = cy;
    }
    sn = 10.0;
    sx *= sn;
    sy *= sn;
#if SHOW_IMG
    for (y = 0; y < cflowmap->rows; y += step)
        for (x = 0; x < cflowmap->cols; x += step)
        {
            CvPoint2D32f fxy = CV_MAT_ELEM(*flow, CvPoint2D32f, y, x);
            cvLine(cflowmap, cvPoint(x,y),
                cvPoint(cvRound(x+fxy.x), cvRound(y+fxy.y)),
                color, 1, 8, 0);
            cvCircle(cflowmap, cvPoint(x,y), 2, color, -1, 8, 0);
        }
#endif
    double v = 2.0 / (_rows * _cols);
    //double v = 0.00004;
    for (y = 0; y < cflowmap->rows; ++y)
        for (x = 0; x < cflowmap->cols; ++x)
        {
            CvPoint2D32f fxy = CV_MAT_ELEM(*flow, CvPoint2D32f, y, x);
            double _x = fxy.x;
            double _y = fxy.y;
            len = sqrt(_x * _x + _y * _y);
            cx += _x * len * v;
            cy += _y * len * v;
            if (len > max_len) max_len = len;
            if (len > 4.0) {
#if 1
              sx += (double)x * (1.0 + len);
              sy += (double)y * (1.0 + len);
              sn += (1.0 + len);
#else
              sx += (double)x;
              sy += (double)y;
              sn += 1.0;
#endif
            }
            lens += len;
            ax += _x;
            ay += _y;
        }

    double surf = _rows * _cols;
    ax /= surf;
    ay /= surf;
    lens /= surf;
    cx = (cx < 0.0 ? 0.0 : cx);
    cy = (cy < 0.0 ? 0.0 : cy);
    cx = (cx > _cols ? _cols : cx);
    cy = (cy > _rows ? _rows : cy);

    sx /= sn;
    sy /= sn;

#if SHOW_IMG
    CvScalar c_color = CV_RGB(0, 255, 0);
    CvScalar s_color = CV_RGB(0, 127, 255);
    cvCircle(cflowmap, cvPoint(cx,cy), 6, c_color, -1, 8, 0);
    cvCircle(cflowmap, cvPoint(sx,sy), 6, s_color, -1, 8, 0);
#endif

    /*
    rx = ((cx / cflowmap->cols) - 1.0) * 2.0;
    ry = ((cy / cflowmap->rows) - 1.0) * 2.0;
    */
    rx = (cx / cflowmap->cols);
    ry = (cy / cflowmap->rows);

#if X_WARP_PTR
    XWarpPointer(display, None, root, 0, 0, 0, 0, rx*screen_w, ry*screen_h);
    XFlush(display);
#endif

    printf("%f %f %f %f %f %f %f\n", lens, ax, ay, rx, ry, sx/_cols, sy/_rows);
    fflush(stdout);
}

int main(int argc, char** argv)
{
    CvMat *prevgray = 0, *gray = 0, *flow = 0, *cflow = 0;
    CvMat *small_gray = 0;
    CvCapture* capture = cvCreateCameraCapture(0);
    if (!capture) return -1;
#if SHOW_IMG
    cvNamedWindow("flow", 1);
#endif
#if X_WARP_PTR
    display = XOpenDisplay(0);
    root = DefaultRootWindow(display);
    screen_number = XDefaultScreen(display);
    screen_w = XDisplayWidth(display, screen_number);
    screen_h = XDisplayHeight(display, screen_number);
#endif
    while (1)
    {
        int firstFrame = gray == 0;
        IplImage* frame = cvQueryFrame(capture);
        if (!frame) break;
        if (!gray) {
            gray = cvCreateMat(frame->height, frame->width, CV_8UC1);
            small_gray = cvCreateMat(frame->height/2, frame->width/2, CV_8UC1);
            prevgray = cvCreateMat(small_gray->rows, small_gray->cols, small_gray->type);
            flow = cvCreateMat(small_gray->rows, small_gray->cols, CV_32FC2);
            cflow = cvCreateMat(small_gray->rows, small_gray->cols, CV_8UC3);
        }
        cvCvtColor(frame, gray, CV_BGR2GRAY);
        cvResize(gray, small_gray, CV_INTER_LINEAR);
        cvFlip(small_gray, small_gray, 1);

  //    {{{ doc for cvCalcOpticalFlowFarneback
  //    cvCalcOpticalFlowFarneback(
  //        const CvArr* prev, const CvArr* next, CvArr* flow,
  //        double pyr_scale, int levels, int winsize,
  //        int iterations, int poly_n, double poly_sigma, int flags)
  //
  //    flow : computed flow image that has the same size as prev and type CV_32FC2.
  //    pyr_scale : parameter, specifying the image scale (<1) to build pyramids
  //                for each image; pyr_scale=0.5 means a classical pyramid, where 
  //                each next layer is twice smaller than the previous one.
  //    levels : number of pyramid layers including the initial image; levels=1 means
  //        that no extra layers are created and only the original images are used.
  //    winsize : averaging window size; larger values increase the algorithm robustness
  //        to image noise and give more chances for fast motion detection, but yield more
  //        blurred motion field.
  //    iterations : number of iterations the algorithm does at each pyramid level.
  //    poly_n : size of the pixel neighborhood used to find polynomial expansion in each pixel;
  //        larger values mean that the image will be approximated with smoother surfaces, yielding
  //        more robust algorithm and more blurred motion field, typically poly_n =5 or 7.
  //    poly_sigma : standard deviation of the Gaussian that is used to smooth derivatives
  //        used as a basis for the polynomial expansion; for poly_n=5, you can set
  //        poly_sigma=1.1, for poly_n=7, a good value would be poly_sigma=1.5.
  //    flags : operation flags that can be a combination of the following:
  //      OPTFLOW_USE_INITIAL_FLOW uses the input flow as an initial flow approximation.
  //      OPTFLOW_FARNEBACK_GAUSSIAN uses the Gaussian \texttt{winsize}\times\texttt{winsize}
  //        filter instead of a box filter of the same size for optical flow estimation;
  //        usually, this option gives z more accurate flow than with a box filter, at the cost
  //        of lower speed; normally, winsize for a Gaussian window should be set to a larger
  //        value to achieve the same level of robustness.
  //    }}}
        if (!firstFrame) {
            double pyr_scale = 0.25; // 0.5
            int levels = 1; // 3
            int winsize = 15; // 15
            int iterations = 1; // 3
            int poly_n = 7; // 5
            double poly_sigma = 1.5; // 1.2
            int flags = 0; // 0
            cvCalcOpticalFlowFarneback(prevgray, small_gray, flow, pyr_scale, levels,
                winsize, iterations, poly_n, poly_sigma, flags);
            cvCvtColor(prevgray, cflow, CV_GRAY2BGR);
            drawOptFlowMap(flow, cflow, 22, CV_RGB(255, 0, 0));
#if SHOW_IMG
            cvShowImage("flow", cflow);
#endif
        }
        if(cvWaitKey(1) >= 0) break;
        {
        CvMat* temp;    
        CV_SWAP(prevgray, small_gray, temp);
        }
    }
    cvReleaseCapture(&capture);
#if X_WARP_PTR
    XCloseDisplay(display);
#endif
    return 0;
}

// vim: fdm=marker
