#include "opencv2/video/tracking.hpp"
#include "opencv2/highgui/highgui.hpp"
#include <stdio.h>
void help()
{
    printf(
        "\n This program demonstrate dense \"Farneback\n optical flow\n"
        "It read from camera 0, and shows how to use and display dense Franeback optical flow\n"
        "Usage: \n"
        "./fback_c \n");

}

void drawOptFlowMap(
        const CvMat* flow, CvMat* cflowmap, int step, CvScalar color)
{
    int x, y;
    for (y = 0; y < cflowmap->rows; y += step)
        for (x = 0; x < cflowmap->cols; x += step)
        {
            CvPoint2D32f fxy = CV_MAT_ELEM(*flow, CvPoint2D32f, y, x);
            cvLine(cflowmap, cvPoint(x,y), cvPoint(cvRound(x+fxy.x), cvRound(y+fxy.y)),
                 color, 1, 8, 0);
            cvCircle(cflowmap, cvPoint(x,y), 2, color, -1, 8, 0);
        }
}

int main(int argc, char** argv)
{
    CvCapture* capture = cvCreateCameraCapture(0);
    CvMat* prevgray = 0, *gray = 0, *flow = 0, *cflow = 0;
    CvMat* half_gray = 0;

    help();
    if (!capture) return -1;
    cvNamedWindow("flow", 1);
    while (1)
    {
        int firstFrame = gray == 0;
        IplImage* frame = cvQueryFrame(capture);
        if (!frame) break;
        if (!gray) {
            gray = cvCreateMat(frame->height, frame->width, CV_8UC1);
            half_gray = cvCreateMat(frame->height/3, frame->width/3, CV_8UC1);
            prevgray = cvCreateMat(half_gray->rows, half_gray->cols, half_gray->type);
            flow = cvCreateMat(half_gray->rows, half_gray->cols, CV_32FC2);
            cflow = cvCreateMat(half_gray->rows, half_gray->cols, CV_8UC3);
        }
        cvCvtColor(frame, gray, CV_BGR2GRAY);
        cvResize(gray, half_gray, CV_INTER_LINEAR);
        cvFlip(half_gray, half_gray, 1);

  //    {{{ doc for cvCalcOpticalFlowFarneback
  //    cvCalcOpticalFlowFarneback(
  //        const CvArr* prev, const CvArr* next, CvArr* flow,
  //        double pyr_scale, int levels, int winsize,
  //        int iterations, int poly_n, double poly_sigma, int flags)
  //
  //    flow : computed flow image that has the same size as prev and type CV_32FC2.
  //    pyr_scale : parameter, specifying the image scale (<1) to build pyramids
  //                for each image; pyr_scale=0.5 means a classical pyramid, where 
  //                each next layer is twice smaller than the previous one.
  //    levels : number of pyramid layers including the initial image; levels=1 means
  //        that no extra layers are created and only the original images are used.
  //    winsize : averaging window size; larger values increase the algorithm robustness
  //        to image noise and give more chances for fast motion detection, but yield more
  //        blurred motion field.
  //    iterations : number of iterations the algorithm does at each pyramid level.
  //    poly_n : size of the pixel neighborhood used to find polynomial expansion in each pixel;
  //        larger values mean that the image will be approximated with smoother surfaces, yielding
  //        more robust algorithm and more blurred motion field, typically poly_n =5 or 7.
  //    poly_sigma : standard deviation of the Gaussian that is used to smooth derivatives
  //        used as a basis for the polynomial expansion; for poly_n=5, you can set
  //        poly_sigma=1.1, for poly_n=7, a good value would be poly_sigma=1.5.
  //    flags : operation flags that can be a combination of the following:
  //      OPTFLOW_USE_INITIAL_FLOW uses the input flow as an initial flow approximation.
  //      OPTFLOW_FARNEBACK_GAUSSIAN uses the Gaussian \texttt{winsize}\times\texttt{winsize}
  //        filter instead of a box filter of the same size for optical flow estimation;
  //        usually, this option gives z more accurate flow than with a box filter, at the cost
  //        of lower speed; normally, winsize for a Gaussian window should be set to a larger
  //        value to achieve the same level of robustness.
  //    }}}
        if (!firstFrame) {
            double pyr_scale = 0.25; // 0.5
            int levels = 1; // 3
            int winsize = 20; // 15
            int iterations = 2; // 3
            int poly_n = 7; // 5
            double poly_sigma = 1.5; // 1.2
            int flags = 0; // 0
            cvCalcOpticalFlowFarneback(prevgray, half_gray, flow, pyr_scale, levels,
                winsize, iterations, poly_n, poly_sigma, flags);
            cvCvtColor(prevgray, cflow, CV_GRAY2BGR);
            drawOptFlowMap(flow, cflow, 22, CV_RGB(255, 0, 0));
            cvShowImage("flow", cflow);
        }
        if(cvWaitKey(1)>=0)
            break;
        {
        CvMat* temp;    
        CV_SWAP(prevgray, half_gray, temp);
        }
    }
    cvReleaseCapture(&capture);
    return 0;
}

// vim: fdm=marker
