/* Copyright (C) 2012, Florent Monnier, all rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

   * Redistribution's of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.

   * Redistribution's in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.

   * The name of the copyright holders may not be used to endorse or promote
     products derived from this software without specific prior written
     permission.

 This software is provided by the copyright holders and contributors "as is" and
 any express or implied warranties, including, but not limited to, the implied
 warranties of merchantability and fitness for a particular purpose are
 disclaimed.  In no event shall the Intel Corporation or contributors be liable
 for any direct, indirect, incidental, special, exemplary, or consequential
 damages (including, but not limited to, procurement of substitute goods or
 services; loss of use, data, or profits; or business interruption) however
 caused and on any theory of liability, whether in contract, strict liability,
 or tort (including negligence or otherwise) arising in any way out of the
 use of this software, even if advised of the possibility of such damage.
*/

#include <opencv2/core/core_c.h>
#include "cv_caml_incs.h"
#include "cv_core_cstub.h"

//int main(int argc, char *argv[])
int main()
{
    //caml_main(argv);
    char arg0[] = "h";
    char arg1[] = "";
    char *argv[3];
    argv[0] = arg0;
    argv[1] = arg1;
    argv[2] = NULL;
    caml_main(argv);
    return 0;
}

CAMLprim value
caml_my_cv_main(value argv)
{
    main();
    return Val_unit;
}

CAMLprim value
caml_cvReleaseImage(value image)
{
    IplImage *img = IplImage_val(image);
    cvReleaseImage(&img);
    return Val_unit;
}

CAMLprim value
caml_IplImage_dims(value image)
{
    CAMLparam1(image);
    CAMLlocal1(ret);
    IplImage *img = IplImage_val(image);
    ret = caml_alloc(2, 0);
    Store_field(ret, 0, Val_long(img->width));
    Store_field(ret, 1, Val_long(img->height));
    CAMLreturn(ret);
}

#define Val_CvMat(mat) ((value)(mat))
#define CvMat_val(mat) ((CvMat *)(mat))

static const int ocaml_table_cv_mat_type[] = {
    CV_8UC1,
    CV_8UC2,
    CV_8UC3,
    CV_8UC4,
    CV_8SC1,
    CV_8SC2,
    CV_8SC3,
    CV_8SC4,
    CV_16UC1,
    CV_16UC2,
    CV_16UC3,
    CV_16UC4,
    CV_16SC1,
    CV_16SC2,
    CV_16SC3,
    CV_16SC4,
    CV_32SC1,
    CV_32SC2,
    CV_32SC3,
    CV_32SC4,
    CV_32FC1,
    CV_32FC2,
    CV_32FC3,
    CV_32FC4,
    CV_64FC1,
    CV_64FC2,
    CV_64FC3,
    CV_64FC4,
};

#define CV_Mat_Type_val(v) \
    (ocaml_table_cv_mat_type[Long_val(v)])


value Val_CV_Mat_Type(int t)
{
    switch (t)
    {
        case CV_8UC1:	return Val_long(0);
        case CV_8UC2:	return Val_long(1);
        case CV_8UC3:	return Val_long(2);
        case CV_8UC4:	return Val_long(3);
        case CV_8SC1:	return Val_long(4);
        case CV_8SC2:	return Val_long(5);
        case CV_8SC3:	return Val_long(6);
        case CV_8SC4:	return Val_long(7);
        case CV_16UC1:	return Val_long(8);
        case CV_16UC2:	return Val_long(9);
        case CV_16UC3:	return Val_long(10);
        case CV_16UC4:	return Val_long(11);
        case CV_16SC1:	return Val_long(12);
        case CV_16SC2:	return Val_long(13);
        case CV_16SC3:	return Val_long(14);
        case CV_16SC4:	return Val_long(15);
        case CV_32SC1:	return Val_long(16);
        case CV_32SC2:	return Val_long(17);
        case CV_32SC3:	return Val_long(18);
        case CV_32SC4:	return Val_long(19);
        case CV_32FC1:	return Val_long(20);
        case CV_32FC2:	return Val_long(21);
        case CV_32FC3:	return Val_long(22);
        case CV_32FC4:	return Val_long(23);
        case CV_64FC1:	return Val_long(24);
        case CV_64FC2:	return Val_long(25);
        case CV_64FC3:	return Val_long(26);
        case CV_64FC4:	return Val_long(27);

        default: caml_failwith("cv_mat_type");
    }
    caml_failwith("cv_mat_type");
}


CAMLprim value
caml_cvCreateMat(value rows, value cols, value type)
{
    CvMat *mat = cvCreateMat(
            Int_val(rows), Int_val(cols), CV_Mat_Type_val(type));
    return Val_CvMat(mat);
}

CAMLprim value
caml_CvMat_dims(value mat)
{
    CAMLparam1(mat);
    CAMLlocal1(ret);
    CvMat *_mat = CvMat_val(mat);
    ret = caml_alloc(2, 0);
    Store_field(ret, 0, Val_long(_mat->cols));
    Store_field(ret, 1, Val_long(_mat->rows));
    CAMLreturn(ret);
}

CAMLprim value
caml_CvMat_get_step(value mat)
{
    CvMat *_mat = CvMat_val(mat);
    return Val_long(_mat->step);
}

CAMLprim value
caml_CvMat_get_type(value mat)
{
    CvMat *_mat = CvMat_val(mat);
    return Val_long(_mat->type);
}

CAMLprim value
caml_CvMat_get_int(value mat, value kind, value i)
{
    CvMat *_mat = CvMat_val(mat);
    unsigned int _i = Long_val(i);
    switch (Int_val(kind))
    {
        case 0: return Val_long(_mat->data.ptr[_i]);
        case 1: return Val_long(_mat->data.s[_i]);
        case 2: return Val_long(_mat->data.i[_i]);
    }
    caml_failwith("cvMat_get_int");
}

CAMLprim value
caml_CvMat_get_float(value mat, value kind, value i)
{
    CvMat *_mat = CvMat_val(mat);
    unsigned int _i = Long_val(i);
    switch (Int_val(kind))
    {
        case 0: return caml_copy_double(_mat->data.fl[_i]);
        case 1: return caml_copy_double(_mat->data.db[_i]);
    }
    caml_failwith("cvMat_get_int");
}

CAMLprim value
caml_CvMat_get_kind(value mat)
{
    int type = CvMat_val(mat)->type;
    return Val_CV_Mat_Type(type);
}

CAMLprim value
caml_cvFlip(value src, value dst, value flip_mode)
{
    cvFlip(
        (CvArr *) src,
        (CvArr *) dst,
        Int_val(flip_mode));
    return Val_unit;
}

CAMLprim value
caml_cvMirror(value src, value flip_mode)
{
    cvFlip(
        (CvArr *) src, NULL,
        Int_val(flip_mode));
    return Val_unit;
}

CAMLprim value
caml_cvLine(
        value img, value pt1, value pt2, value color,
        value thickness, value line_type, value shift)
{
    CvPoint _pt1 = cvPoint(Int_val(Field(pt1,0)), Int_val(Field(pt1,1)));
    CvPoint _pt2 = cvPoint(Int_val(Field(pt2,0)), Int_val(Field(pt2,1)));
    CvScalar _color = cvScalar(
        Double_val(Field(color,0)),
        Double_val(Field(color,1)),
        Double_val(Field(color,2)),
        Double_val(Field(color,3)));
    cvLine(
        (CvArr *) img,
        _pt1, _pt2, _color,
        Int_val(thickness),
        Int_val(line_type),
        Int_val(shift));
    return Val_unit;
}

CAMLprim value
caml_cvLine_bc(value *argv, int argn)
{
    return caml_cvLine(
            argv[0], argv[1], argv[2], argv[3],
            argv[4], argv[5], argv[6]);
}

/* vim: set sw=4 sts=4 ts=4 et: */
