/* Copyright (C) 2012, Florent Monnier, all rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

   * Redistribution's of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.

   * Redistribution's in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.

   * The name of the copyright holders may not be used to endorse or promote
     products derived from this software without specific prior written
     permission.

 This software is provided by the copyright holders and contributors "as is" and
 any express or implied warranties, including, but not limited to, the implied
 warranties of merchantability and fitness for a particular purpose are
 disclaimed.  In no event shall the Intel Corporation or contributors be liable
 for any direct, indirect, incidental, special, exemplary, or consequential
 damages (including, but not limited to, procurement of substitute goods or
 services; loss of use, data, or profits; or business interruption) however
 caused and on any theory of liability, whether in contract, strict liability,
 or tort (including negligence or otherwise) arising in any way out of the
 use of this software, even if advised of the possibility of such damage.
*/

#include <opencv2/highgui/highgui_c.h>
#include "cv_caml_incs.h"
#include "cv_core_cstub.h"

#define Val_CvCapture(cap) ((value)(cap))
#define CvCapture_val(cap) ((CvCapture *)(cap))

#include <stdio.h>
CAMLprim value
caml_cvCreateCameraCapture(value index)
{
    printf("## before\n");
    fflush(stdout);
    CvCapture *capture = cvCreateCameraCapture(Int_val(index));
    printf("## after\n");
    fflush(stdout);
    if (!capture) caml_failwith("cvCreateCameraCapture");
    return Val_CvCapture(capture);
}

CAMLprim value
caml_cvReleaseCapture(value capture)
{
    CvCapture *cap = CvCapture_val(capture);
    cvReleaseCapture(&cap);
    return Val_unit;
}

CAMLprim value
caml_cvQueryFrame(value capture)
{
    IplImage *frame = cvQueryFrame(CvCapture_val(capture));
    if (!frame) caml_failwith("cvQueryFrame");
    return Val_IplImage(frame);
}

static const int ocaml_table_cv_window_flags[] = {
    CV_WINDOW_NORMAL,
    CV_WINDOW_AUTOSIZE,
    CV_WINDOW_FULLSCREEN,
    CV_WINDOW_FREERATIO,
    CV_WINDOW_KEEPRATIO,
};

#define CV_Window_flags_val(v) \
    (ocaml_table_cv_window_flags[Long_val(v)])

CAMLprim value
caml_cvNamedWindow(value name, value flags)
{
    int r = cvNamedWindow(String_val(name), CV_Window_flags_val(flags));
    if (!r) caml_failwith("cvNamedWindow");
    return Val_unit;
}

CAMLprim value
caml_cvResizeWindow(value name, value width, value height)
{
    cvResizeWindow(String_val(name), Int_val(width), Int_val(height));
    return Val_unit;
}

CAMLprim value
caml_cvMoveWindow(value name, value x, value y)
{
    cvMoveWindow(String_val(name), Int_val(x), Int_val(y));
    return Val_unit;
}

CAMLprim value
caml_cvDestroyWindow(value name)
{
    cvDestroyWindow(String_val(name));
    return Val_unit;
}

CAMLprim value
caml_cvShowImage(value name, value image)
{
    IplImage *img = IplImage_val(image);
    cvShowImage(String_val(name), img);
    return Val_unit;
}

CAMLprim value
caml_cvWaitKey(value delay)
{
    int k = cvWaitKey(Int_val(delay));
    return Val_int(k);
}

/* vim: set sw=4 sts=4 ts=4 et: */
