/* Copyright (C) 2012, Florent Monnier, all rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

   * Redistribution's of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.

   * Redistribution's in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.

   * The name of the copyright holders may not be used to endorse or promote
     products derived from this software without specific prior written
     permission.

 This software is provided by the copyright holders and contributors "as is" and
 any express or implied warranties, including, but not limited to, the implied
 warranties of merchantability and fitness for a particular purpose are
 disclaimed.  In no event shall the Intel Corporation or contributors be liable
 for any direct, indirect, incidental, special, exemplary, or consequential
 damages (including, but not limited to, procurement of substitute goods or
 services; loss of use, data, or profits; or business interruption) however
 caused and on any theory of liability, whether in contract, strict liability,
 or tort (including negligence or otherwise) arising in any way out of the
 use of this software, even if advised of the possibility of such damage.
*/

#include <opencv2/imgproc/types_c.h>
#include <opencv2/imgproc/imgproc_c.h>
#include "cv_caml_incs.h"

static const int ocaml_table_color_conversion[] = {
    CV_BGR2BGRA,
    CV_RGB2RGBA,
    CV_BGRA2BGR,
    CV_RGBA2RGB,
    CV_BGR2RGBA,
    CV_RGB2BGRA,
    CV_RGBA2BGR,
    CV_BGRA2RGB,
    CV_BGR2RGB,
    CV_RGB2BGR,
    CV_BGRA2RGBA,
    CV_RGBA2BGRA,
    CV_BGR2GRAY,
    CV_RGB2GRAY,
    CV_GRAY2BGR,
    CV_GRAY2RGB,
    CV_GRAY2BGRA,
    CV_GRAY2RGBA,
    CV_BGRA2GRAY,
    CV_RGBA2GRAY,
    CV_BGR2BGR565,
    CV_RGB2BGR565,
    CV_BGR5652BGR,
    CV_BGR5652RGB,
    CV_BGRA2BGR565,
    CV_RGBA2BGR565,
    CV_BGR5652BGRA,
    CV_BGR5652RGBA,
    CV_GRAY2BGR565,
    CV_BGR5652GRAY,
    CV_BGR2BGR555,
    CV_RGB2BGR555,
    CV_BGR5552BGR,
    CV_BGR5552RGB,
    CV_BGRA2BGR555,
    CV_RGBA2BGR555,
    CV_BGR5552BGRA,
    CV_BGR5552RGBA,
    CV_GRAY2BGR555,
    CV_BGR5552GRAY,
    CV_BGR2XYZ,
    CV_RGB2XYZ,
    CV_XYZ2BGR,
    CV_XYZ2RGB,
    CV_BGR2YCrCb,
    CV_RGB2YCrCb,
    CV_YCrCb2BGR,
    CV_YCrCb2RGB,
    CV_BGR2HSV,
    CV_RGB2HSV,
    CV_BGR2Lab,
    CV_RGB2Lab,
    CV_BayerBG2BGR,
    CV_BayerGB2BGR,
    CV_BayerRG2BGR,
    CV_BayerGR2BGR,
    CV_BayerBG2RGB,
    CV_BayerGB2RGB,
    CV_BayerRG2RGB,
    CV_BayerGR2RGB,
    CV_BGR2Luv,
    CV_RGB2Luv,
    CV_BGR2HLS,
    CV_RGB2HLS,
    CV_HSV2BGR,
    CV_HSV2RGB,
    CV_Lab2BGR,
    CV_Lab2RGB,
    CV_Luv2BGR,
    CV_Luv2RGB,
    CV_HLS2BGR,
    CV_HLS2RGB,
    CV_BayerBG2BGR_VNG,
    CV_BayerGB2BGR_VNG,
    CV_BayerRG2BGR_VNG,
    CV_BayerGR2BGR_VNG,
    CV_BayerBG2RGB_VNG,
    CV_BayerGB2RGB_VNG,
    CV_BayerRG2RGB_VNG,
    CV_BayerGR2RGB_VNG,
    CV_BGR2HSV_FULL,
    CV_RGB2HSV_FULL,
    CV_BGR2HLS_FULL,
    CV_RGB2HLS_FULL,
    CV_HSV2BGR_FULL,
    CV_HSV2RGB_FULL,
    CV_HLS2BGR_FULL,
    CV_HLS2RGB_FULL,
    CV_LBGR2Lab,
    CV_LRGB2Lab,
    CV_LBGR2Luv,
    CV_LRGB2Luv,
    CV_Lab2LBGR,
    CV_Lab2LRGB,
    CV_Luv2LBGR,
    CV_Luv2LRGB,
    CV_BGR2YUV,
    CV_RGB2YUV,
    CV_YUV2BGR,
    CV_YUV2RGB,
    CV_BayerBG2GRAY,
    CV_BayerGB2GRAY,
    CV_BayerRG2GRAY,
    CV_BayerGR2GRAY,
    CV_YUV420i2RGB,
    CV_YUV420i2BGR,
    CV_YUV420sp2RGB,
    CV_YUV420sp2BGR,
    CV_COLORCVT_MAX,
};

#define Color_conversion_val(v) \
    (ocaml_table_color_conversion[Long_val(v)])

CAMLprim value
caml_cvCvtColor(value src, value dst, value code)
{
    cvCvtColor(
        (CvArr *) src,
        (CvArr *) dst,
        Color_conversion_val(code));
    return Val_unit;
}


static const int ocaml_table_interpolation[] = {
    CV_INTER_NN,
    CV_INTER_LINEAR,
    CV_INTER_CUBIC,
    CV_INTER_AREA,
    CV_INTER_LANCZOS4,
};

#define Interpolation_val(v) \
    (ocaml_table_interpolation[Long_val(v)])

CAMLprim value
caml_cvResize(value src, value dst, value interpolation)
{
    cvResize(
        (CvArr *) src,
        (CvArr *) dst,
        Interpolation_val(interpolation));
    return Val_unit;
}

/* vim: set sw=4 sts=4 ts=4 et: */
