/* Copyright (C) 2012, Florent Monnier, all rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

   * Redistribution's of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.

   * Redistribution's in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.

   * The name of the copyright holders may not be used to endorse or promote
     products derived from this software without specific prior written
     permission.

 This software is provided by the copyright holders and contributors "as is" and
 any express or implied warranties, including, but not limited to, the implied
 warranties of merchantability and fitness for a particular purpose are
 disclaimed.  In no event shall the Intel Corporation or contributors be liable
 for any direct, indirect, incidental, special, exemplary, or consequential
 damages (including, but not limited to, procurement of substitute goods or
 services; loss of use, data, or profits; or business interruption) however
 caused and on any theory of liability, whether in contract, strict liability,
 or tort (including negligence or otherwise) arising in any way out of the
 use of this software, even if advised of the possibility of such damage.
*/

//#include <opencv2/core/core.hpp>
//#include <opencv2/highgui/highgui.hpp>
#include <opencv2/opencv.hpp>
#include "cv_caml_incs.hpp"
using namespace cv;

// VideoCapture();
// VideoCapture(const string& filename);
// VideoCapture(int device);

#define Val_VideoCapture(cap) ((value)(cap))
#define VideoCapture_val(cap) ((VideoCapture *)(cap))

#define Val_Mat(mat) ((value)(mat))
#define Mat_val(mat) ((Mat *)(mat))

CAMLextern_C
value caml_VideoCapture_new(value device)
{
    VideoCapture *cap = new VideoCapture(Int_val(device));
    if (!cap->isOpened()) caml_failwith("new_VideoCapture");
    return Val_VideoCapture(cap);
}

CAMLextern_C
value caml_VideoCapture_delete(value cap)
{
    delete VideoCapture_val(cap);
    return Val_unit;
}

CAMLextern_C
value caml_VideoCapture_acq(value cap, value mat)
{
    *VideoCapture_val(cap) >> *Mat_val(mat);
    return Val_unit;
}

CAMLextern_C
value caml_namedWindow(value winname)
{
    namedWindow(String_val(winname), 1);
    return Val_unit;
}

CAMLextern_C
value caml_Mat_new(value unit)
{
    Mat *mat = new Mat();
    return Val_Mat(mat);
}

CAMLextern_C
value caml_imshow(value winname, value mat)
{
    imshow(String_val(winname), *(Mat_val(mat)));
    return Val_unit;
}

CAMLextern_C
value caml_waitKey(value delay)
{
    int k = waitKey(Int_val(delay));
    return Val_int(k);
}

//void cvtColor( InputArray src, OutputArray dst, int code, int dstCn=0 );

// vim: sw=4 sts=4 ts=4 et
